@@ -131,11 +131,11 @@ def get_files(downloads, callback=None, sizes=None):
     else:
         sizes = [None] * len(downloads)
     # Download all the files, blocking until we get them all.
-    if config.service.timeout <= timedelta():
+    if config.system.timeout <= timedelta():
         # E.g. -1s or 0s or 0d etc.
         timeout = None
     else:
-        timeout = config.service.timeout.total_seconds()
+        timeout = config.system.timeout.total_seconds()
     # Repack the downloads so that the _get_one_file() function will be called
     # with the proper set of arguments.
     args = [(timeout, url, dst, size, callback)
@@ -150,7 +150,7 @@ def get_files(downloads, callback=None, sizes=None):
         # context manager.
         for url, path in downloads:
             stack.callback(safe_remove, path)
-        with ThreadPoolExecutor(max_workers=config.service.threads) as tpe:
+        with ThreadPoolExecutor(max_workers=config.system.threads) as tpe:
             # All we need to do is iterate over the returned generator in
             # order to complete all the requests.  There's really nothing to
             # return.  Either all the files got downloaded, or they didn't.
@@ -171,8 +171,8 @@ def get_files(downloads, callback=None, sizes=None):
             # fails, rather than os.remove()'ing them.
             try:
                 list(tpe.map(_get_one_file, args))
-            except (HTTPError, URLError, CertificateError):
-                raise FileNotFoundError
+            except (HTTPError, URLError, CertificateError) as error:
+                raise FileNotFoundError(str(error)) from error
             # Check all the signed files.  First, grab the blacklists file if
             # there is one available.
         # Everything's fine so do *not* delete the downloaded files.
