@@ -23,6 +23,7 @@ __all__ = [
 
 import os
 import math
+#import pickle
 import shutil
 import hashlib
 import logging
@@ -31,7 +32,6 @@ from collections import deque
 from contextlib import ExitStack
 from functools import partial
 from itertools import islice
-from operator import itemgetter
 from systemimage.candidates import get_candidates, iter_path
 from systemimage.channel import Channels
 from systemimage.config import config
@@ -45,6 +45,7 @@ from urllib.parse import urljoin
 
 log = logging.getLogger('systemimage')
 COMMASPACE = ', '
+COLON = ':'
 
 
 class ChecksumError(Exception):
@@ -62,19 +63,30 @@ def _download_feedback(url, dst, bytes_read, size):
 
 
 class State:
-    def __init__(self, callback=None):
+    def __init__(self, callback=None, candidate_filter=None):
         # Variables which manage state transitions.
         self._next = deque()
-        self._next.append(self._get_blacklist_1)
         self._debug_step = 1
         self._callback = (_download_feedback if callback is None else callback)
+        self._filter = candidate_filter
         # Variables which represent things we've learned.
         self.blacklist = None
         self.channels = None
         self.index = None
-        self.candidates = None
         self.winner = None
         self.files = []
+        # See if there is a state file to unpickle.
+        first_step = self._get_blacklist_1
+        ## try:
+        ##     with open(config.system.state_file, 'rb') as fp:
+        ##         self.blacklist = pickle.load(fp)
+        ##         self.winner = pickle.load(fp)
+        ##         pickle_date = pickle.load(fp)
+        ##         # XXX check for stale file.
+        ##     first_step = self._download_files
+        ## except FileNotFoundError:
+        ##     pass
+        self._next.append(first_step)
 
     def __iter__(self):
         return self
@@ -240,21 +252,22 @@ class State:
             log.info('Local channels file: {}', channels_path)
             with open(channels_path, encoding='utf-8') as fp:
                 self.channels = Channels.from_json(fp.read())
-        # The next step will depend on whether there is a device keyring
-        # available or not.  If there is, download and verify it now.
+        # Locate the index file for the channel/device.
         try:
-            device = getattr(
-                # This device's channel.
-                getattr(self.channels, config.system.channel),
-                config.device)
-        except AttributeError:
-            log.info('no matching channel/device: {}/{}',
-                     config.system.channel, config.device)
-            # Either our channel or device isn't described in the
-            # channels.json file, so there's nothing more to do.
+            channel = self.channels[config.channel]
+        except KeyError:
+            log.info('no matching channel: {}', config.channel)
+            return
+        log.info('got channel: {}', config.channel)
+        try:
+            device = channel.devices[config.device]
+        except KeyError:
+            log.info('no matching device: {}', config.device)
             return
         log.info('found channel/device entry: {}/{}',
-                 config.system.channel, config.device)
+                 config.channel, config.device)
+        # The next step will depend on whether there is a device keyring
+        # available or not.  If there is, download and verify it now.
         keyring = getattr(device, 'keyring', None)
         if keyring:
             self._next.append(partial(self._get_device_keyring, keyring))
@@ -269,6 +282,44 @@ class State:
             self.blacklist)
         # We don't need to set the next action because it's already been done.
 
+    def _get_master_key(self):
+        """Try to get and validate a new image master key.
+
+        If there isn't one, throw a SignatureError.
+        """
+        try:
+            log.info('Getting the image master key')
+            # The image signing key must be signed by the archive master.
+            get_keyring(
+                'image-master', 'gpg/image-master.tar.xz',
+                'archive-master', self.blacklist)
+        except (FileNotFoundError, SignatureError, KeyringError):
+            # No valid image master key could be found.  Don't chain this
+            # exception.
+            log.error('No valid imaging master key found')
+            raise SignatureError from None
+        # Retry the previous step.
+        log.info('Installing new image master key to: {}',
+                 config.gpg.image_master)
+        self._next.appendleft(self._get_blacklist_2)
+
+    def _get_signing_key(self):
+        """Try to get and validate a new image signing key.
+
+        If there isn't one, throw a SignatureError.
+        """
+        try:
+            # The image signing key must be signed by the image master.
+            get_keyring(
+                'image-signing', 'gpg/image-signing.tar.xz', 'image-master',
+                self.blacklist)
+        except (FileNotFoundError, SignatureError, KeyringError):
+            # No valid image signing key could be found.  Don't chain this
+            # exception.
+            raise SignatureError from None
+        # Retry the previous step.
+        self._next.appendleft(self._get_channel)
+
     def _get_index(self, index):
         """Get and verify the index.json file."""
         index_url = urljoin(config.service.https_base, index)
@@ -291,6 +342,8 @@ class State:
             ctx = stack.enter_context(
                 Context(*keyrings, blacklist=self.blacklist))
             if not ctx.verify(asc_path, index_path):
+                log.error('index.json signature failure: {} {}',
+                          index_path, asc_path)
                 raise SignatureError(index_path)
             # The signature was good.
             with open(index_path, encoding='utf-8') as fp:
@@ -300,15 +353,31 @@ class State:
     def _calculate_winner(self):
         """Given an index, calculate the paths and score a winner."""
         # Store these as attributes for debugging and testing.
-        self.candidates = get_candidates(self.index, config.build_number)
-        self.winner = config.hooks.scorer().choose(self.candidates)
+        candidates = get_candidates(self.index, config.build_number)
+        if self._filter is not None:
+            candidates = self._filter(candidates)
+        self.winner = config.hooks.scorer().choose(candidates)
         # If there is no winning upgrade candidate, then there's nothing more
         # to do.  We can skip everything between downloading the files and
         # doing the reboot.
         if len(self.winner) > 0:
-            self._next.append(self._download_files)
+            winning_path = [str(image.version) for image in self.winner]
+            log.info('Upgrade path is {}'.format(COLON.join(winning_path)))
+            self._next.append(self._persist)
         else:
-            log.info('No update available.')
+            log.info('Already up-to-date')
+
+    def _persist(self):
+        """Persist enough state to resume right to downloading files."""
+        ## with open(config.system.state_file, 'wb') as fp:
+        ##     # Things we need to pickle:
+        ##     # - the location of our blacklist file
+        ##     # - the set of calculated winners
+        ##     # - the current datetime
+        ##     pickle.dump(self.blacklist, fp)
+        ##     pickle.dump(self.winner, fp)
+        ##     pickle.dump(datetime.now(), fp)
+        self._next.append(self._download_files)
 
     def _download_files(self):
         """Download and verify all the winning upgrade path's files."""
@@ -374,44 +443,6 @@ class State:
         # for the upgrader.
         self._next.append(self._move_files)
 
-    def _get_master_key(self):
-        """Try to get and validate a new image master key.
-
-        If there isn't one, throw a SignatureError.
-        """
-        try:
-            log.info('Getting the image master key')
-            # The image signing key must be signed by the archive master.
-            get_keyring(
-                'image-master', 'gpg/image-master.tar.xz',
-                'archive-master', self.blacklist)
-        except (FileNotFoundError, SignatureError, KeyringError):
-            # No valid image master key could be found.  Don't chain this
-            # exception.
-            log.error('No valid imaging master key found')
-            raise SignatureError from None
-        # Retry the previous step.
-        log.info('Installing new image master key to: {}',
-                 config.gpg.image_master)
-        self._next.appendleft(self._get_blacklist_2)
-
-    def _get_signing_key(self):
-        """Try to get and validate a new image signing key.
-
-        If there isn't one, throw a SignatureError.
-        """
-        try:
-            # The image signing key must be signed by the image master.
-            get_keyring(
-                'image-signing', 'gpg/image-signing.tar.xz', 'image-master',
-                self.blacklist)
-        except (FileNotFoundError, SignatureError, KeyringError):
-            # No valid image signing key could be found.  Don't chain this
-            # exception.
-            raise SignatureError from None
-        # Retry the previous step.
-        self._next.appendleft(self._get_channel)
-
     def _move_files(self):
         # The upgrader already has the archive-master, so we don't need to
         # copy it.  The image-master, image-signing, and device-signing (if
