@@ -28,8 +28,8 @@ from contextlib import ExitStack
 from systemimage.api import Cancel, Mediator
 from systemimage.config import config
 from systemimage.testing.helpers import (
-    copy, make_http_server, setup_index, setup_keyring_txz, setup_keyrings,
-    sign, temporary_directory, testable_configuration)
+    configuration, copy, make_http_server, setup_index, setup_keyring_txz,
+    setup_keyrings, sign, temporary_directory, touch_build)
 
 
 class TestAPI(unittest.TestCase):
@@ -83,53 +83,49 @@ class TestAPI(unittest.TestCase):
             os.path.join(self._serverdir, 'stable', 'nexus7',
                          'device-signing.tar.xz'))
 
-    @testable_configuration
+    @configuration
     def test_update_available(self):
         # Because our build number is lower than the latest available in the
         # index file, there is an update available.
         self._setup_keyrings()
-        mediator = Mediator()
-        self.assertTrue(mediator.check_for_update())
+        update = Mediator().check_for_update()
+        self.assertTrue(update.is_available)
 
-    @testable_configuration
+    @configuration
     def test_update_available_version(self):
         # An update is available.  What's the target version number?
         self._setup_keyrings()
-        mediator = Mediator()
-        updates = mediator.check_for_update()
-        self.assertEqual(updates.version, 20130600)
+        update = Mediator().check_for_update()
+        self.assertEqual(update.version, '20130600')
 
-    @testable_configuration
+    @configuration
     def test_no_update_available_version(self):
         # No update is available, so the target version number is zero.
         self._setup_keyrings()
-        with open(config.system.build_file, 'w', encoding='utf-8') as fp:
-            print(20130600, file=fp)
-        mediator = Mediator()
-        updates = mediator.check_for_update()
-        self.assertEqual(updates.version, 0)
+        touch_build(20130600)
+        update = Mediator().check_for_update()
+        self.assertFalse(update.is_available)
+        self.assertEqual(update.version, '')
 
-    @testable_configuration
+    @configuration
     def test_no_update_available_at_latest(self):
         # Because our build number is equal to the latest available in the
         # index file, there is no update available.
         self._setup_keyrings()
-        with open(config.system.build_file, 'w', encoding='utf-8') as fp:
-            print(20130600, file=fp)
-        mediator = Mediator()
-        self.assertFalse(mediator.check_for_update())
+        touch_build(20130600)
+        update = Mediator().check_for_update()
+        self.assertFalse(update.is_available)
 
-    @testable_configuration
+    @configuration
     def test_no_update_available_newer(self):
         # Because our build number is higher than the latest available in the
         # index file, there is no update available.
         self._setup_keyrings()
-        with open(config.system.build_file, 'w', encoding='utf-8') as fp:
-            print(20130700, file=fp)
-        mediator = Mediator()
-        self.assertFalse(mediator.check_for_update())
+        touch_build(20130700)
+        update = Mediator().check_for_update()
+        self.assertFalse(update.is_available)
 
-    @testable_configuration
+    @configuration
     def test_get_details(self):
         # Get the details of an available update.
         self._setup_keyrings()
@@ -140,9 +136,8 @@ class TestAPI(unittest.TestCase):
         sign(self.index_path, 'device-signing.gpg')
         setup_index('index_14.json', self._serverdir, 'device-signing.gpg')
         # Get the descriptions.
-        mediator = Mediator()
-        update = mediator.check_for_update()
-        self.assertTrue(update)
+        update = Mediator().check_for_update()
+        self.assertTrue(update.is_available)
         self.assertEqual(update.size, 180009)
         self.assertEqual(len(update.descriptions), 3)
         # The first contains the descriptions for the full update.
@@ -165,8 +160,8 @@ class TestAPI(unittest.TestCase):
             'description-xx_CC': 'This hyar is the delta B.2',
             })
 
-    @testable_configuration
-    def test_complete_update(self):
+    @configuration
+    def test_download(self):
         # After checking that an update is available, complete the update, but
         # don't reboot.
         self._setup_keyrings()
@@ -179,7 +174,7 @@ class TestAPI(unittest.TestCase):
             got_reboot = True
         with unittest.mock.patch(
                 'systemimage.reboot.Reboot.reboot', reboot_mock):
-            mediator.complete_update()
+            mediator.download()
         # No reboot got issued.
         self.assertFalse(got_reboot)
         # But the command file did get written, and all the files are present.
@@ -218,7 +213,7 @@ unmount system
             'blacklist.tar.xz.asc',
             ]))
 
-    @testable_configuration
+    @configuration
     def test_reboot(self):
         # Run the intermediate steps, and finish with a reboot.
         self._setup_keyrings()
@@ -231,36 +226,16 @@ unmount system
         with unittest.mock.patch(
                 'systemimage.reboot.Reboot.reboot', reboot_mock):
             mediator.check_for_update()
-            mediator.complete_update()
+            mediator.download()
             self.assertFalse(got_reboot)
             mediator.reboot()
             self.assertTrue(got_reboot)
 
-    @testable_configuration
+    @configuration
     def test_cancel(self):
         # When we get to the step of downloading the files, cancel it.
         self._setup_keyrings()
         mediator = Mediator()
         mediator.check_for_update()
         mediator.cancel()
-        self.assertRaises(Cancel, mediator.complete_update)
-
-    @testable_configuration
-    def test_reboot_after_cancel(self):
-        # Trying to reboot after the download has been canceled raises the
-        # Cancel exception.
-        self._setup_keyrings()
-        mediator = Mediator()
-        mediator.check_for_update()
-        # Let's say that the update got canceled after all the files were
-        # downloaded.
-        mediator.complete_update()
-        mediator.cancel()
-        self.assertRaises(Cancel, mediator.reboot)
-
-    @testable_configuration
-    def test_get_build_number(self):
-        with open(config.system.build_file, 'w', encoding='utf-8') as fp:
-            print(20130701, file=fp)
-        mediator = Mediator()
-        self.assertEqual(mediator.get_build_number(), 20130701)
+        self.assertRaises(Cancel, mediator.download)
