@@ -18,9 +18,12 @@
 __all__ = [
     'TestCommandFileDelta',
     'TestCommandFileFull',
+    'TestDailyProposed',
     'TestFileOrder',
-    'TestRebootingState',
+    'TestPersistence',
+    'TestRebooting',
     'TestState',
+    'TestStateNewChannelsFormat',
     ]
 
 
@@ -29,13 +32,15 @@ import hashlib
 import unittest
 
 from contextlib import ExitStack
+from datetime import datetime, timezone
 from subprocess import CalledProcessError
 from systemimage.config import config
 from systemimage.gpg import SignatureError
 from systemimage.state import State
+from systemimage.testing.demo import DemoDevice
 from systemimage.testing.helpers import (
-    copy, get_index, make_http_server, setup_index, setup_keyring_txz,
-    setup_keyrings, sign, temporary_directory, testable_configuration)
+    configuration, copy, data_path, get_index, make_http_server, setup_index,
+    setup_keyring_txz, setup_keyrings, sign, temporary_directory, touch_build)
 from unittest.mock import patch
 
 
@@ -59,7 +64,7 @@ class TestState(unittest.TestCase):
     def tearDown(self):
         self._stack.close()
 
-    @testable_configuration
+    @configuration
     def test_first_signature_fails_get_new_image_signing_key(self):
         # The first time we check the channels.json file, the signature fails,
         # because it's blacklisted.  Everything works out in the end though
@@ -89,8 +94,7 @@ class TestState(unittest.TestCase):
         # signed correctly, new state transitions will be added to re-aquire a
         # new image signing key.
         state = State()
-        next(state)
-        next(state)
+        state.run_thru('get_channel')
         # Where we would expect a channels object, there is none.
         self.assertIsNone(state.channels)
         # Just to prove that the image signing key is going to change, let's
@@ -106,10 +110,10 @@ class TestState(unittest.TestCase):
         # valid channels object.
         sign(self._channels_path, 'image-signing.gpg')
         next(state)
-        self.assertEqual(state.channels.stable.nexus7.index,
+        self.assertEqual(state.channels.stable.devices.nexus7.index,
                          '/stable/nexus7/index.json')
 
-    @testable_configuration
+    @configuration
     def test_first_signature_fails_get_bad_image_signing_key(self):
         # The first time we check the channels.json file, the signature fails.
         # We try to get the new image signing key, but it is bogus.
@@ -126,8 +130,7 @@ class TestState(unittest.TestCase):
         # signed correctly, new state transitions will be added to re-aquire a
         # new image signing key.
         state = State()
-        next(state)
-        next(state)
+        state.run_thru('get_channel')
         # Where we would expect a channels object, there is none.
         self.assertIsNone(state.channels)
         # Just to prove that the image signing key is not going to change,
@@ -141,7 +144,7 @@ class TestState(unittest.TestCase):
         with open(config.gpg.image_signing, 'rb') as fp:
             self.assertEqual(checksum, hashlib.md5(fp.read()).digest())
 
-    @testable_configuration
+    @configuration
     def test_bad_system_image_master_exposed_by_blacklist(self):
         # The blacklist is signed by the image master key.  If the blacklist's
         # signature is bad, the state machine will attempt to download a new
@@ -173,7 +176,7 @@ class TestState(unittest.TestCase):
         next(state)
         self.assertEqual(os.path.basename(state.blacklist), 'blacklist.tar.xz')
 
-    @testable_configuration
+    @configuration
     def test_bad_system_image_master_new_one_is_no_better(self):
         # The blacklist is signed by the system image master key.  If the
         # blacklist's signature is bad, the state machine will attempt to
@@ -204,7 +207,7 @@ class TestState(unittest.TestCase):
         with open(config.gpg.image_master, 'rb') as fp:
             self.assertEqual(checksum, hashlib.md5(fp.read()).digest())
 
-    @testable_configuration
+    @configuration
     def test_image_master_is_missing(self):
         # The system only comes pre-seeded with the archive master public
         # keyring.  All others are downloaded.
@@ -222,7 +225,7 @@ class TestState(unittest.TestCase):
         # Now the image master key exists.
         self.assertTrue(os.path.exists(config.gpg.image_master))
 
-    @testable_configuration
+    @configuration
     def test_image_master_is_missing_with_blacklist(self):
         # The system only comes pre-seeded with the archive master public
         # keyring.  All others are downloaded.  This time there is a
@@ -244,7 +247,7 @@ class TestState(unittest.TestCase):
         # Now the image master key exists.
         self.assertTrue(os.path.exists(config.gpg.image_master))
 
-    @testable_configuration
+    @configuration
     def test_image_signing_is_missing(self):
         # The system only comes pre-seeded with the archive master public
         # keyring.  All others are downloaded.
@@ -268,13 +271,12 @@ class TestState(unittest.TestCase):
         self.assertFalse(os.path.exists(config.gpg.image_master))
         self.assertFalse(os.path.exists(config.gpg.image_signing))
         state = State()
-        next(state)
-        next(state)
+        state.run_thru('get_channel')
         # Now the image master and signing keys exist.
         self.assertTrue(os.path.exists(config.gpg.image_master))
         self.assertTrue(os.path.exists(config.gpg.image_signing))
 
-    @testable_configuration
+    @configuration
     def test_downloaded_image_signing_is_still_bad(self):
         # LP: #1191979: Let's say there's a blacklist.tar.xz file but it is
         # not signed with the system image master key.  The state machine will
@@ -296,14 +298,16 @@ class TestState(unittest.TestCase):
         # Run the state machine three times:
         # blacklist -(sig fail)-> get master -> blacklist (sig fail)
         state = State()
-        next(state)
-        next(state)
+        state.run_thru('get_master_key')
         self.assertRaises(SignatureError, next, state)
 
 
 class _StateTestsBase(unittest.TestCase):
     # Must override in base classes.
     INDEX_FILE = None
+    CHANNEL_FILE = None
+    CHANNEL = None
+    DEVICE = None
 
     # For more detailed output.
     maxDiff = None
@@ -319,12 +323,18 @@ class _StateTestsBase(unittest.TestCase):
                 self._serverdir, 8943, 'cert.pem', 'key.pem'))
             self._stack.push(make_http_server(self._serverdir, 8980))
             # Set up the server files.
-            copy('channels_06.json', self._serverdir, 'channels.json')
+            assert self.CHANNEL_FILE is not None, (
+                'Subclasses must set CHANNEL_FILE')
+            copy(self.CHANNEL_FILE, self._serverdir, 'channels.json')
             sign(os.path.join(self._serverdir, 'channels.json'),
                  'image-signing.gpg')
+            assert self.CHANNEL is not None, 'Subclasses must set CHANNEL'
+            assert self.DEVICE is not None, 'Subclasses must set DEVICE'
             index_path = os.path.join(
-                self._serverdir, 'stable', 'nexus7', 'index.json')
+                self._serverdir, self.CHANNEL, self.DEVICE, 'index.json')
             head, tail = os.path.split(index_path)
+            assert self.INDEX_FILE is not None, (
+                'Subclasses must set INDEX_FILE')
             copy(self.INDEX_FILE, head, tail)
             sign(index_path, 'device-signing.gpg')
             setup_index(self.INDEX_FILE, self._serverdir, 'device-signing.gpg')
@@ -356,7 +366,7 @@ class _StateTestsBase(unittest.TestCase):
         setup_keyring_txz(
             'device-signing.gpg', 'image-signing.gpg',
             dict(type='device-signing'),
-            os.path.join(self._serverdir, 'stable', 'nexus7',
+            os.path.join(self._serverdir, self.CHANNEL, self.DEVICE,
                          'device-signing.tar.xz'))
 
 
@@ -364,8 +374,11 @@ class TestRebooting(_StateTestsBase):
     """Test various state transitions leading to a reboot."""
 
     INDEX_FILE = 'index_13.json'
+    CHANNEL_FILE = 'channels_06.json'
+    CHANNEL = 'stable'
+    DEVICE = 'nexus7'
 
-    @testable_configuration
+    @configuration
     def test_keyrings_copied_to_upgrader_paths(self):
         # The following keyrings get copied to system paths that the upgrader
         # consults:
@@ -374,12 +387,6 @@ class TestRebooting(_StateTestsBase):
         # * image-signing.tar.xz{,.asc}  - cache partition
         # * device-signing.tar.xz{,.asc} - cache partition (if one exists)
         self._setup_keyrings()
-        # Run the state machine enough times to download all the keyrings and
-        # data files, then to move the files into place just before a reboot
-        # is issued.  Steps preceded by * are steps that fail.
-        # *get blacklist/get master -> get channels/signing
-        # -> get device signing -> get index -> calculate winner
-        # -> download files -> move files
         cache_dir = config.updater.cache_partition
         data_dir = config.updater.data_partition
         blacklist_path = os.path.join(data_dir, 'blacklist.tar.xz')
@@ -403,9 +410,14 @@ class TestRebooting(_StateTestsBase):
                     cache_dir, os.path.basename(filerec.signature))
                 self.assertFalse(os.path.exists(path))
                 self.assertFalse(os.path.exists(asc))
+        # Run the state machine enough times to download all the keyrings and
+        # data files, then to move the files into place just before a reboot
+        # is issued.  Steps preceded by * are steps that fail.
+        # *get blacklist/get master -> get channels/signing
+        # -> get device signing -> get index -> calculate winner
+        # -> download files -> move files
         state = State()
-        for i in range(7):
-            next(state)
+        state.run_thru('move_files')
         # All of the keyrings and .asc files are found.
         self.assertTrue(os.path.exists(blacklist_path))
         self.assertTrue(os.path.exists(master_path))
@@ -424,7 +436,7 @@ class TestRebooting(_StateTestsBase):
                 self.assertTrue(os.path.exists(path))
                 self.assertTrue(os.path.exists(asc))
 
-    @testable_configuration
+    @configuration
     def test_reboot_issued(self):
         # The reboot gets issued.
         self._setup_keyrings()
@@ -433,27 +445,25 @@ class TestRebooting(_StateTestsBase):
         self.assertEqual(mock.call_args[0][0],
                          ['/sbin/reboot', '-f', 'recovery'])
 
-
-    @testable_configuration
+    @configuration
     def test_no_update_available_no_reboot(self):
         # LP: #1202915.  If there's no update available, running the state
         # machine to completion should not result in a reboot.
         self._setup_keyrings()
         # Hack the current build number so that no update is available.
-        with open(config.system.build_file, 'w', encoding='utf-8') as fp:
-            print(20250000, file=fp)
+        touch_build(20250000)
         with patch('systemimage.reboot.Reboot.reboot') as mock:
             list(State())
         self.assertEqual(mock.call_count, 0)
 
     @unittest.skipIf(os.getuid() == 0, 'This test would actually reboot!')
-    @testable_configuration
+    @configuration
     def test_reboot_fails(self):
         # The reboot fails, e.g. because we are not root.
         self._setup_keyrings()
         self.assertRaises(CalledProcessError, list, State())
 
-    @testable_configuration
+    @configuration
     def test_run_until(self):
         # It is possible to run the state machine either until some specific
         # state is completed, or it runs to the end.
@@ -486,8 +496,11 @@ class TestRebooting(_StateTestsBase):
 
 class TestCommandFileFull(_StateTestsBase):
     INDEX_FILE = 'index_13.json'
+    CHANNEL_FILE = 'channels_06.json'
+    CHANNEL = 'stable'
+    DEVICE = 'nexus7'
 
-    @testable_configuration
+    @configuration
     def test_full_command_file(self):
         # A full update's command file gets properly filled.
         self._setup_keyrings()
@@ -510,14 +523,16 @@ unmount system
 
 class TestCommandFileDelta(_StateTestsBase):
     INDEX_FILE = 'index_15.json'
+    CHANNEL_FILE = 'channels_06.json'
+    CHANNEL = 'stable'
+    DEVICE = 'nexus7'
 
-    @testable_configuration
+    @configuration
     def test_delta_command_file(self):
         # A delta update's command file gets properly filled.
         self._setup_keyrings()
         # Set the current build number so a delta update will work.
-        with open(config.system.build_file, 'w', encoding='utf-8') as fp:
-            print(20120100, file=fp)
+        touch_build(20120100)
         State().run_until('reboot')
         path = os.path.join(config.updater.cache_partition, 'ubuntu_command')
         with open(path, 'r', encoding='utf-8') as fp:
@@ -536,15 +551,17 @@ unmount system
 
 class TestFileOrder(_StateTestsBase):
     INDEX_FILE = 'index_16.json'
+    CHANNEL_FILE = 'channels_06.json'
+    CHANNEL = 'stable'
+    DEVICE = 'nexus7'
 
-    @testable_configuration
+    @configuration
     def test_file_order(self):
         # Updates are applied sorted first by image positional order, then
         # within the image by the 'order' key.
         self._setup_keyrings()
         # Set the current build number so a delta update will work.
-        with open(config.system.build_file, 'w', encoding='utf-8') as fp:
-            print(20120100, file=fp)
+        touch_build(20120100)
         State().run_until('reboot')
         path = os.path.join(config.updater.cache_partition, 'ubuntu_command')
         with open(path, 'r', encoding='utf-8') as fp:
@@ -566,3 +583,163 @@ update h.txt h.txt.asc
 update i.txt i.txt.asc
 unmount system
 """)
+
+
+@unittest.skip('persistence is temporarily disabled')
+class TestPersistence(_StateTestsBase):
+    """Test the State object's persistence."""
+
+    INDEX_FILE = 'index_16.json'
+    CHANNEL_FILE = 'channels_06.json'
+    CHANNEL = 'stable'
+    DEVICE = 'nexus7'
+
+    @configuration
+    def test_pickle_file(self):
+        # Run the state machine through the 'persist' state.  Create a new
+        # state object which restores the persisted state.
+        self._setup_keyrings()
+        self.assertFalse(os.path.exists(config.system.state_file))
+        state = State()
+        self.assertIsNone(state.winner)
+        state.run_thru('persist')
+        self.assertIsNotNone(state.winner)
+        self.assertTrue(os.path.exists(config.system.state_file))
+        state = State()
+        self.assertIsNotNone(state.winner)
+
+    @configuration
+    def test_no_update_no_pickle_file(self):
+        # If there's no update, there's no state file.
+        self._setup_keyrings()
+        touch_build(20250000)
+        self.assertFalse(os.path.exists(config.system.state_file))
+        state = State()
+        self.assertIsNone(state.winner)
+        state.run_thru('persist')
+        self.assertEqual(state.winner, [])
+        self.assertFalse(os.path.exists(config.system.state_file))
+        state = State()
+        self.assertIsNone(state.winner)
+
+
+class TestDailyProposed(_StateTestsBase):
+    """Test that the daily-proposed channel works as expected."""
+
+    INDEX_FILE = 'index_13.json'
+    CHANNEL_FILE = 'channels_07.json'
+    CHANNEL = 'daily-proposed'
+    DEVICE = 'grouper'
+
+    @configuration
+    def test_daily_proposed_channel(self):
+        # Resolve the index.json path for a channel with a dash in it.
+        self._setup_keyrings()
+        state = State()
+        self._stack.enter_context(
+            patch('systemimage.state.config.channel', 'daily-proposed'))
+        self._stack.enter_context(
+            patch('systemimage.state.config.hooks.device', DemoDevice))
+        state.run_thru('get_index')
+        self.assertEqual(state.index.global_.generated_at,
+                         datetime(2013, 8, 1, 8, 1, tzinfo=timezone.utc))
+
+    @configuration
+    def test_bogus_channel(self):
+        # Try and fail to resolve the index.json path for a non-existent
+        # channel with a dash in it.
+        self._setup_keyrings()
+        state = State()
+        self._stack.enter_context(
+            patch('systemimage.state.config.channel', 'daily-testing'))
+        self._stack.enter_context(
+            patch('systemimage.state.config.hooks.device', DemoDevice))
+        state.run_thru('get_index')
+        self.assertIsNone(state.index)
+
+
+class TestVersionedProposed(_StateTestsBase):
+    INDEX_FILE = 'index_13.json'
+    CHANNEL_FILE = 'channels_08.json'
+    CHANNEL = '14.04-proposed'
+    DEVICE = 'grouper'
+
+    @configuration
+    def test_version_proposed_channel(self):
+        # Resolve the index.json path for a channel with a dash and a dot in
+        # it.
+        self._setup_keyrings()
+        state = State()
+        self._stack.enter_context(
+            patch('systemimage.state.config.channel', '14.04-proposed'))
+        self._stack.enter_context(
+            patch('systemimage.state.config.hooks.device', DemoDevice))
+        state.run_thru('get_index')
+        self.assertEqual(state.index.global_.generated_at,
+                         datetime(2013, 8, 1, 8, 1, tzinfo=timezone.utc))
+
+
+class TestFilters(_StateTestsBase):
+    INDEX_FILE = 'index_15.json'
+    CHANNEL_FILE = 'channels_06.json'
+    CHANNEL = 'stable'
+    DEVICE = 'nexus7'
+
+    @configuration
+    def test_filter_none(self):
+        # With no filter, we get the unadulterated candidate paths.
+        self._setup_keyrings()
+        touch_build(20120100)
+        state = State()
+        state.run_thru('calculate_winner')
+        self.assertEqual(len(state.winner), 1)
+
+    @configuration
+    def test_filter_1(self):
+        # The state machine can use a filter to come up with a different set
+        # of candidate upgrade paths.  In this case, no candidates.
+        self._setup_keyrings()
+        touch_build(20120100)
+        def filter_out_everything(candidates):
+            return []
+        state = State(candidate_filter=filter_out_everything)
+        state.run_thru('calculate_winner')
+        self.assertEqual(len(state.winner), 0)
+
+
+class TestStateNewChannelsFormat(_StateTestsBase):
+    CHANNEL_FILE = 'channels_09.json'
+    CHANNEL = 'saucy'
+    DEVICE = 'manta'
+    INDEX_FILE = 'index_21.json'
+
+    @configuration
+    def test_full_reboot(self, ini_file):
+        # Test that state transitions through reboot work for the new channel
+        # format.  Also check that the right files get moved into place.
+        self._stack.enter_context(patch('systemimage.device.check_output',
+                                        return_value='manta'))
+        config.load(data_path('channel_04.ini'), override=True)
+        self._setup_keyrings()
+        state = State()
+        state.run_until('reboot')
+        path = os.path.join(config.updater.cache_partition, 'ubuntu_command')
+        with open(path, 'r', encoding='utf-8') as fp:
+            command = fp.read()
+        self.assertMultiLineEqual(command, """\
+load_keyring image-master.tar.xz image-master.tar.xz.asc
+load_keyring image-signing.tar.xz image-signing.tar.xz.asc
+load_keyring device-signing.tar.xz device-signing.tar.xz.asc
+mount system
+update 6.txt 6.txt.asc
+update 7.txt 7.txt.asc
+update 5.txt 5.txt.asc
+unmount system
+""")
+        got_reboot = False
+        def reboot_mock(self):
+            nonlocal got_reboot
+            got_reboot = True
+        with patch('systemimage.reboot.Reboot.reboot', reboot_mock):
+            list(state)
+        self.assertTrue(got_reboot)
