@@ -26,15 +26,12 @@ import unittest
 from contextlib import ExitStack
 from systemimage.candidates import get_candidates
 from systemimage.config import config
-from systemimage.download import get_files
 from systemimage.gpg import SignatureError
 from systemimage.helpers import temporary_directory
-from systemimage.scores import WeightedScorer
 from systemimage.state import ChecksumError, State
 from systemimage.testing.helpers import (
-    copy, get_index, make_http_server, makedirs, setup_index,
-    setup_keyring_txz, setup_keyrings, sign, test_data_path,
-    testable_configuration)
+    configuration, copy, make_http_server, setup_index, setup_keyring_txz,
+    setup_keyrings, sign, touch_build)
 
 
 class TestWinnerDownloads(unittest.TestCase):
@@ -70,44 +67,38 @@ class TestWinnerDownloads(unittest.TestCase):
     def tearDown(self):
         self._stack.close()
 
-    @testable_configuration
+    @configuration
     def test_calculate_candidates(self):
         # Calculate the candidate paths.
         setup_keyrings()
         state = State()
-        # Run the state machine 4 times to get the candidates and winner.
-        # (blacklist -> channel -> index -> calculate)
-        for i in range(4):
-            next(state)
-        # Set the build number.
-        with open(config.system.build_file, 'wt', encoding='utf-8') as fp:
-            print(20120100, file=fp)
+        # Run the state machine until we get an index file.
+        state.run_until('calculate_winner')
+        candidates = get_candidates(state.index, 20120100)
         # There are three candidate upgrade paths.
-        self.assertEqual(len(state.candidates), 3)
+        self.assertEqual(len(candidates), 3)
         descriptions = []
-        for image in state.candidates[0]:
+        for image in candidates[0]:
             # There's only one description per image so order doesn't matter.
             descriptions.extend(image.descriptions.values())
         self.assertEqual(descriptions, ['Full A', 'Delta A.1', 'Delta A.2'])
         descriptions = []
-        for image in state.candidates[1]:
+        for image in candidates[1]:
             # There's only one description per image so order doesn't matter.
             descriptions.extend(image.descriptions.values())
         self.assertEqual(descriptions, ['Full B', 'Delta B.1', 'Delta B.2'])
         descriptions = []
-        for image in state.candidates[2]:
+        for image in candidates[2]:
             # There's only one description per image so order doesn't matter.
             descriptions.extend(image.descriptions.values())
         self.assertEqual(descriptions, ['Full C', 'Delta C.1'])
 
-    @testable_configuration
+    @configuration
     def test_calculate_winner(self):
         # Calculate the winning upgrade path.
         setup_keyrings()
         state = State()
-        # Set the build number.
-        with open(config.system.build_file, 'wt', encoding='utf-8') as fp:
-            print(20120100, file=fp)
+        touch_build(20120100)
         # Run the state machine 4 times to get the candidates and winner.
         # (blacklist -> channel -> index -> calculate)
         for i in range(4):
@@ -119,18 +110,14 @@ class TestWinnerDownloads(unittest.TestCase):
             descriptions.extend(image.descriptions.values())
         self.assertEqual(descriptions, ['Full B', 'Delta B.1', 'Delta B.2'])
 
-    @testable_configuration
+    @configuration
     def test_download_winners(self):
         # Check that all the winning path's files are downloaded.
         setup_keyrings()
         state = State()
-        # Set the build number.
-        with open(config.system.build_file, 'wt', encoding='utf-8') as fp:
-            print(20120100, file=fp)
-        # Run the state machine 5 times to get the files.
-        # (blacklist -> channel -> index -> calculate -> download)
-        for i in range(5):
-            next(state)
+        touch_build(20120100)
+        # Run the state machine until we download the files.
+        state.run_thru('download_files')
         # The B path files contain their checksums.
         def assert_file_contains(filename, contents):
             path = os.path.join(config.system.tempdir, filename)
@@ -148,7 +135,7 @@ class TestWinnerDownloads(unittest.TestCase):
         assert_file_contains('d.txt', 'fed')
         assert_file_contains('c.txt', 'edc')
 
-    @testable_configuration
+    @configuration
     def test_download_winners_signed_by_device_key(self):
         # Check that all the winning path's files are downloaded, even when
         # they are signed by the device key instead of the image signing
@@ -168,15 +155,10 @@ class TestWinnerDownloads(unittest.TestCase):
         sign(os.path.join(self._serverdir, self._indexpath),
              'device-signing.gpg')
         setup_index('index_12.json', self._serverdir, 'device-signing.gpg')
-        # Set the build number.
-        with open(config.system.build_file, 'wt', encoding='utf-8') as fp:
-            print(20120100, file=fp)
-        # Because there's a device key, run the state machine 6 times to get
-        # the files.
-        # (blacklist -> channel -> index -> devicekey -> calculate -> download)
+        touch_build(20120100)
+        # Run the state machine until we download the files.
         state = State()
-        for i in range(6):
-            next(state)
+        state.run_thru('download_files')
         # The B path files contain their checksums.
         def assert_file_contains(filename, contents):
             path = os.path.join(config.system.tempdir, filename)
@@ -194,7 +176,7 @@ class TestWinnerDownloads(unittest.TestCase):
         assert_file_contains('d.txt', 'fed')
         assert_file_contains('c.txt', 'edc')
 
-    @testable_configuration
+    @configuration
     def test_download_winners_signed_by_signing_key_with_device_key(self):
         # Check that all the winning path's files are downloaded, even when
         # they are signed by the device key instead of the image signing
@@ -213,15 +195,10 @@ class TestWinnerDownloads(unittest.TestCase):
              'device-signing.gpg')
         # All the downloadable files are now signed with the image signing key.
         setup_index('index_12.json', self._serverdir, 'image-signing.gpg')
-        # Set the build number.
-        with open(config.system.build_file, 'wt', encoding='utf-8') as fp:
-            print(20120100, file=fp)
-        # Because there's a device key, run the state machine 6 times to get
-        # the files.
-        # (blacklist -> channel -> index -> devicekey -> calculate -> download)
+        touch_build(20120100)
+        # Run the state machine until we download the files.
         state = State()
-        for i in range(6):
-            next(state)
+        state.run_thru('download_files')
         # The B path files contain their checksums.
         def assert_file_contains(filename, contents):
             path = os.path.join(config.system.tempdir, filename)
@@ -239,7 +216,7 @@ class TestWinnerDownloads(unittest.TestCase):
         assert_file_contains('d.txt', 'fed')
         assert_file_contains('c.txt', 'edc')
 
-    @testable_configuration
+    @configuration
     def test_download_winners_bad_checksums(self):
         # Similar to the various good paths, except because the checksums are
         # wrong in index_10.json, we'll get a error when downloading.
@@ -249,17 +226,13 @@ class TestWinnerDownloads(unittest.TestCase):
         setup_index('index_10.json', self._serverdir, 'image-signing.gpg')
         setup_keyrings()
         state = State()
-        # Set the build number.
-        with open(config.system.build_file, 'wt', encoding='utf-8') as fp:
-            print(20120100, file=fp)
-        # Run the state machine 4 times to get prepped.
-        # (blacklist -> channel -> index -> calculate)
+        touch_build(20120100)
+        # Run the state machine until we're prepped to download
+        state.run_until('download_files')
         # Now try to download the files and get the error.
-        for i in range(4):
-            next(state)
         self.assertRaises(ChecksumError, next, state)
 
-    @testable_configuration
+    @configuration
     def test_download_winners_signed_by_wrong_key(self):
         # There is a device key, but the image files are signed by the image
         # signing key, which according to the spec means the files are not
@@ -278,17 +251,10 @@ class TestWinnerDownloads(unittest.TestCase):
              'device-signing.gpg')
         # All the downloadable files are now signed with a bogus key.
         setup_index('index_12.json', self._serverdir, 'spare.gpg')
-        # Set the build number.
-        with open(config.system.build_file, 'wt', encoding='utf-8') as fp:
-            print(20120100, file=fp)
-        # Because there's a device key, run the state machine 6 times to get
-        # the files.  However, since we want to catch the exception on step 6,
-        # run it just 5 times now.
-        #
-        # (blacklist -> channel -> index -> devicekey -> calculate -> download)
+        touch_build(20120100)
+        # Run the state machine until just before we download the files.
         state = State()
-        for i in range(5):
-            next(state)
+        state.run_until('download_files')
         # The next state transition will fail because of the missing signature.
         self.assertRaises(SignatureError, next, state)
         # There are no downloaded files.
@@ -297,21 +263,17 @@ class TestWinnerDownloads(unittest.TestCase):
                        if os.path.splitext(filename)[1] == '.txt')
         self.assertEqual(len(txtfiles), 0)
 
-    @testable_configuration
+    @configuration
     def test_no_download_winners_with_missing_signature(self):
         # If one of the download files is missing a signature, none of the
         # files get downloaded and get_files() fails.
         setup_keyrings()
         state = State()
-        # Set the build number.
-        with open(config.system.build_file, 'wt', encoding='utf-8') as fp:
-            print(20120100, file=fp)
+        touch_build(20120100)
         # Remove a signature.
         os.remove(os.path.join(self._serverdir, '6/7/8.txt.asc'))
-        # Run the state machine 4 times to calculate the winning path.
-        # (blacklist -> channel -> index -> calculate [-> download])
-        for i in range(4):
-            next(state)
+        # Run the state machine to calculate the winning path.
+        state.run_until('download_files')
         # The next state transition will fail because of the missing signature.
         self.assertRaises(FileNotFoundError, next, state)
         # There are no downloaded files.
@@ -320,21 +282,17 @@ class TestWinnerDownloads(unittest.TestCase):
                        if os.path.splitext(filename)[1] == '.txt')
         self.assertEqual(len(txtfiles), 0)
 
-    @testable_configuration
+    @configuration
     def test_no_download_winners_with_bad_signature(self):
         # If one of the download files has a bad a signature, none of the
         # files get downloaded and get_files() fails.
         setup_keyrings()
         state = State()
-        # Set the build number.
-        with open(config.system.build_file, 'wt', encoding='utf-8') as fp:
-            print(20120100, file=fp)
+        touch_build(20120100)
         # Break a signature
         sign(os.path.join(self._serverdir, '6', '7', '8.txt'), 'spare.gpg')
-        # Run the state machine 4 times to calculate the winning path.
-        # (blacklist -> channel -> index -> calculate [-> download])
-        for i in range(4):
-            next(state)
+        # Run the state machine to calculate the winning path.
+        state.run_until('download_files')
         # The next state transition will fail because of the missing signature.
         self.assertRaises(SignatureError, next, state)
         # There are no downloaded files.
